<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioOrderPrefixer.php';
require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioPayments.php';

class MontonioCardPayments extends MontonioPayments
{
    const METHOD_CODE = 'MontonioCardPayments';
    const DEFAULT_LOGO_URL = 'https://public.montonio.com/images/logos/visa-mc-ap-gp.png';

    public static function getConfig()
    {
        $options = array(
            array(
                'type' => 'switch',
                'label' => 'Enable Montonio Card Payments',
                'desc' => 'When enabled, show Montonio Card Payments as a payment option at checkout',
                'is_bool' => true,
                'name' => 'MONTONIO_CARD_ENABLED',
                'values' => array(
                    array(
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => 'Yes'
                    ),
                    array(
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => 'No'
                    )
                ),
            ),
            array(
                'type' => 'switch',
                'label' => 'Show logo?',
                'desc' => 'You can turn this off if you don\'t want to show Montonio Card logo at checkout',
                'is_bool' => true,
                'name' => 'MONTONIO_CARD_SHOW_LOGO',
                'values' => array(
                    array(
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => 'Yes'
                    ),
                    array(
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => 'No'
                    )
                ),
            ),
            array(
                'type' => 'textarea',
                'label' => 'Custom CSS for Checkout page',
                'name' => 'MONTONIO_CARD_PAYMENT_HANDLE_CSS',
                'autoload_rte' => false,
            )
        );
        return $options;
    }

    const CONFIGDEFAULTS = array(
        'MONTONIO_CARD_ENABLED' => false,
        'MONTONIO_CARD_CREATE_ORDER' => 'before',
        'MONTONIO_CARD_SHOW_LOGO' => '1',
        'MONTONIO_CARD_STYLE' => 'grid_logos',
        'MONTONIO_CARD_DEFAULT_COUNTRY' => 'EE',
        'MONTONIO_CARD_PAYMENT_HANDLE_CSS' => '',
        'MONTONIO_CARD_AUTOMATICALLY_CHANGE_COUNTRY' => 'manual',
        'MONTONIO_CARD_AUTOMATICALLY_SELECT_METHOD' => 'no',
        'MONTONIO_CARD_TRANSLATE_COUNTRY_DROPDOWN' => 'english',
    );

    const TRANSLATIONS = array(
        'title' => 'Card Payment',
        'description' => 'Pay with your credit or debit card via Montonio.',
    );

    /**
     * @return bool
     */
    public static function isEnabled()
    {
        return Configuration::get('MONTONIO_CARD_ENABLED') === '1';
    }

    /**
     * @param $module
     * @return PaymentOption|null
     */
    public static function getCheckoutPaymentOption($module)
    {
        /**
         * Check if option is enabled and supports selected currency
         */
        if (
            !self::isEnabled() ||
            !$module->isCurrencySupported(self::METHOD_CODE)
        ) {
            return null;
        }

        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user presses Place Order
         */
        $formAction = Context::getContext()->link->getModuleLink(
            $module->name,
            'validation',
            array('method' => self::METHOD_CODE),
            true
        );

        /**
         * Assign the url form action to the template var $action
         */
        Context::getContext()->smarty->assign([
            'action' => $formAction,
            'montonioPaymentsPaymentHandleCSS' => Configuration::get('MONTONIO_CARD_PAYMENT_HANDLE_CSS'),
        ]);

        /**
         * Get Description for Payment method
         */
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsCheckout.php';
        $checkout = new montonio_payments_Checkout();
        $checkout->set_description($module->montonioCardPaymentsDesc);

        /**
         * Set the initial country for dropdown
         */
        $defaultCountry = Configuration::get('MONTONIO_CARD_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'EE';
        }

        $optionChangeCountryMode = Configuration::get('MONTONIO_CARD_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {
            $customerLanguage = Language::getIsoById((int)Context::getContext()->cookie->id_lang);
            $regionsByLanguage = array(
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'sw' => 'DE',
                'de' => 'DE'
            );
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        $checkout->set_preferred_country($defaultCountry);

        $checkout->set_payment_handle_style(Configuration::get('MONTONIO_CARD_STYLE'));

        $checkout->set_attempt_to_select_payment(
            Configuration::get('MONTONIO_CARD_AUTOMATICALLY_SELECT_METHOD', 'no')
        );

        $description = '<p>' . $module->montonioCardPaymentsDesc . '</p>';

        $title = $module->montonioCardPaymentsTitle;

        /**
         *  Load form template to be displayed in the checkout step
         */
        $paymentForm = $module->fetch('module:montonio/views/templates/hook/payment_options.tpl');

        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($module->displayName)
            ->setCallToActionText($title)
            ->setAdditionalInformation($description)
            ->setAction($formAction)
            ->setForm($paymentForm);

        /**
         * Get Logo for Payment method and
         * assign the description to the template var $logo
         */
        $logo = Configuration::get('MONTONIO_CARD_SHOW_LOGO');
        if ($logo != '0') {
            $newOption->setLogo(self::DEFAULT_LOGO_URL);
        }
        return $newOption;
    }

    /**
     * Prepare the data to be submitted to Montonio Payments
     *
     * @param object $context The execution context
     * @param object $module Montonio class instance
     *
     * @return array          The data that will be sent to Montonio
     */
    public static function preparePaymentData($context, $module)
    {

        $cart = $context->cart;
        $customer = $context->customer;

        // add Prefix to Cart ID if needed
        $storePrefix = Configuration::get('MONTONIO_ORDER_PREFIX');
        $cartId = (string)$cart->id;

        $notificationUrlArgs = [
            'method' => self::METHOD_CODE,
            'id_cart' => $cartId,
            'id_module' => $module->id,
            'key' => $customer->secure_key,
            'is_notification' => 'yes',
        ];

        $returnUrlArgs = [
            'method' => self::METHOD_CODE,
            'id_cart' => $cartId,
            'id_module' => $module->id,
            'key' => $customer->secure_key,
        ];

        // Shipping modules' Integration ********************************************************
        $carrier = new Carrier($cart->id_carrier);
        $carrierInfo = null;
        $carrierName = $carrier->external_module_name;
        if ($carrierName) {
            $key = $carrierName . '_' . $cart->id_address_delivery;
            if (isset($context->cookie->$key)) {
                $carrierInfo = $context->cookie->$key;
            }
        }

        if ($carrierInfo) {
            $notificationUrlArgs['extra_data'] = [$key => $carrierInfo];
            $returnUrlArgs['extra_data'] = [$key => $carrierInfo];
        }
        // /Shipping modules' Integration *******************************************************

        $moduleName = $module->name;

        /**
         * Generate order before redirect
         */
        $createOrderBefore = true;
        if (Configuration::get('MONTONIO_CARD_CREATE_ORDER') === 'before') {
            $createOrderBefore = true;
        }
        if ($createOrderBefore) {
            $module->validateOrder(
                (int)$cart->id,
                Configuration::get('PS_OS_MONTONIO'),
                $cart->getOrderTotal(),
                $module->displayName . ' (' .self::METHOD_CODE . ', Cart ID: ' . (int)$cart->id . ')',
                'Order Created with ' . self::METHOD_CODE,
                NULL,
                (int)$context->currency->id,
                false,
                $context->customer->secure_key
            );
        }

        if (isset($module->currentOrder)) {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $module->currentOrder);
            $order = new Order((int)$module->currentOrder);
        } else {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);
            $order = $context->cart;
        }

        $merchantRef = $orderRef . ((Configuration::get('MONTONIO_ENVIRONMENT') === 'sandbox') ? '_' . rand() : '');

        //can change on sandbox
        $returnUrlArgs['id_cart'] = $merchantRef;
        $notificationUrlArgs['id_cart'] = $merchantRef;

        $idCurrency = $order->id_currency;
        $paymentCurrency = new Currency($idCurrency); // should contain iso_code
        $currencyIsoCode = $paymentCurrency->iso_code;

        $orderTotal = isset($order->total_paid) ? $order->total_paid : $order->getOrderTotal();

        $langId = Language::getIsoById((int)Context::getContext()->cookie->id_lang);
        $availableLocales = self::AVAILABLE_LOCALES;
        $config = [
            'merchantReference' => $merchantRef,
            'returnUrl' => $context->link->getModuleLink($moduleName, 'callback', $returnUrlArgs),
            'notificationUrl' => $context->link->getModuleLink($moduleName, 'callback', $notificationUrlArgs),
            'currency' => $currencyIsoCode,
            'grandTotal' => (float)$orderTotal,
            'locale' => (isset($availableLocales[$langId])) ? $availableLocales[$langId] : 'en',
        ];

        return self::getPayload($config, $order, $module, $context);
    }

    /**
     * Prepare data transfer object
     *
     * @param array $config
     * @param Cart|Order $order
     * @param object $module
     * @param $context
     * @return array
     */
    public static function getPayload(array $config, $order, $module, $context)
    {
        $payload = $config;

        if($context->customer->isLogged()) {
            $customer = new Customer((int) $order->id_customer);
        } else {
            //order object does not have id_guest , cart has
            $id_guest = isset($order->id_guest) ? $order->id_guest : $order->id_customer;
            $customer = new Customer((int)$id_guest);
        }

        $shippingAddress = new Address((int)$order->id_address_delivery);
        $payload['shippingAddress'] = self::getAddressPayload($shippingAddress, $customer->email);

        $billingAddress = new Address((int)$order->id_address_invoice);
        $payload['billingAddress'] = self::getAddressPayload($billingAddress, $customer->email);

        foreach($order->getProducts() as $product) {
            $name = isset($product['name']) ? $product['name'] : $product['product_name'];
            $quantity = isset($product['quantity']) ? $product['quantity'] : $product['product_quantity'];
            $payload['lineItems'][] = [
                'name'       => (string)Tools::safeOutput($name),
                'quantity'   => (int)Tools::safeOutput($quantity),
                'finalPrice' => (float)Tools::safeOutput($product['total_wt']),
            ];
        }

        $defaultCountry = Configuration::get('MONTONIO_CARD_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'PL';
        }
        $optionChangeCountryMode = Configuration::get('MONTONIO_CARD_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {
            $customerLanguage = Language::getIsoById((int)Context::getContext()->cookie->id_lang);
            $regionsByLanguage = [
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'de' => 'DE'
            ];
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        if (Tools::getValue('preselectedCountry') && Tools::getValue('preselectedCountry') != '') {
            $defaultCountry = Tools::getValue('preselectedCountry');
        }

        $payload['payment'] = [
            'method' => 'cardPayments',
            'methodDisplay' => (string)Tools::safeOutput($module->montonioCardPaymentsTitle),
            'methodOptions' => [
                // paymentReference
                // Structured payment reference number.
                // This is a standardised reference number used for accounting purposes and will be validated by banks.
                // Do not include it in the data if you do not use reference numbers to link payments.
                //'paymentReference' => '',
                'paymentDescription' => 'Payment for order '. $payload['merchantReference'],
                'preferredCountry' => $defaultCountry,
            ],
            'amount' => $payload['grandTotal'],
            'currency' => $payload['currency']
        ];

        return $payload;
    }

    /**
     * Get smarty data for legacy checkout option (Prestashop 1.6)
     *
     * @param array$params The params from hookPayment hook
     * @return array       The smarty params
     */
    public static function getLegacyPaymentOption($module)
    {

        $enabled = self::isEnabled() && $module->isCurrencySupported('MontonioCardPayments');

        /**
         * Check if option is enabled and supports selected currency
         */
        if (
            !self::isEnabled() ||
            !$module->isCurrencySupported(self::METHOD_CODE)
        ) {
            return null;
        }
        /**
         * Get Description for Payment method
         */
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsCheckout.php';
        $checkout = new Montonio_Payments_Checkout();
        $checkout->set_description($module->montonioCardPaymentsDesc);
        $checkout->setPaymentLinkBase(Context::getContext()->link->getModuleLink('montonio', 'validation', array(
            'method' => 'MontonioCardPayments',
        )));

        /**
         * Set the initial country for dropdown
         */
        $defaultCountry = Configuration::get('MONTONIO_CARD_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'EE';
        }
        $customerLanguage = Language::getIsoById((int) Context::getContext()->cookie->id_lang);
        $optionChangeCountryMode = Configuration::get('MONTONIO_CARD_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {

            // TODO: Check if not exists
            $regionsByLanguage = array(
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'de' => 'DE'
            );
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        $checkout->set_preferred_country($defaultCountry);

        /**
         * Translate the dropdown if needed
         */
        $optionTranslateDropdown = Configuration::get('MONTONIO_CARD_TRANSLATE_COUNTRY_DROPDOWN');
        if ($optionTranslateDropdown == 'translated') {
            $checkout->set_regions(array(
                'EE' => $module->textRegionEE,
                'LT' => $module->textRegionLT,
                'LV' => $module->textRegionLV,
                'FI' => $module->textRegionFI,
                'PL' => $module->textRegionPL,
                'DE' => $module->textRegionDE
            ));
        }

        $checkout->set_payment_handle_style(Configuration::get('MONTONIO_CARD_STYLE'));
        $checkout->set_payment_handle_extra_css(''); // TODO

        /*$bankList = (Configuration::get('MONTONIO_ENVIRONMENT') == 'sandbox')
            ? 'MONTONIO_PAYMENTS_CURRENT_BANKS_SANDBOX'
            : 'MONTONIO_PAYMENTS_CURRENT_BANKS';

        $checkout->set_banklist(Configuration::get($bankList));*/
        $banksUI = $checkout->get_description_html();

        $paymentHandleCss = Configuration::get('MONTONIO_CARD_PAYMENT_HANDLE_CSS');

        $montonioPaymentsData = array(
            'enabled'            => $enabled,
            'title'              => $module->montonioCardPaymentsTitle,
            'description'        => $module->montonioCardPaymentsDesc,
            'logo_url'           => self::DEFAULT_LOGO_URL,
            'show_logo'          => Configuration::get('MONTONIO_CARD_SHOW_LOGO') == '1',
            'enabled'            => Configuration::get('MONTONIO_CARD_ENABLED') == '1',
            'paymentHandleStyle' => Configuration::get('MONTONIO_CARD_STYLE'),
            'banksUI'            => $banksUI,
        );

        if ($paymentHandleCss != '' && !empty($paymentHandleCss)) {
            $montonioPaymentsData['paymentHandleCss'] = $paymentHandleCss;
        }

        return $montonioPaymentsData;
    }

}
