<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioOrderPrefixer.php';
require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioSplitSDK.php';

class MontonioSplit
{
    const SPLIT_METHOD_COMBINED = 'Pay later, no fees';

    const MIN_ORDER_TOTAL = '25';

    const LOGOS = array(
        'ee' => 'https://public.montonio.com/logo/financing/ee-split-3.png',
        'lv' => 'https://public.montonio.com/logo/financing/lt-split-3.png',
        'lt' => 'https://public.montonio.com/logo/financing/lv-split-3.png',
        'en' => 'https://public.montonio.com/logo/financing/en-split-3.png'
    );
    const METHOD_CODE = 'MontonioSplit';

    public static function getConfig()
    {
        $options = array(
            array(
                'type'    => 'switch',
                'label'   => 'Enable Montonio Split',
                'desc'    => 'When enabled, show Montonio Split as a payment option at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_SPLIT_ENABLED',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'    => 'switch',
                'label'   => 'Show logo?',
                'desc'    => 'You can turn this off if you don\'t want to show Montonio logo at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_SPLIT_SHOW_LOGO',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'     => 'text',
                'label'    => 'Min. order amount',
                'desc'     => 'The minimum order amount required for Montonio Split in checkout. Should not be under 25€.',
                'name'     => 'MONTONIO_SPLIT_MIN_ORDER_AMOUNT',
                'size'     => 10,
                'required' => true,
            ),
        );

        return $options;
    }

    const CONFIGDEFAULTS = array(
        'MONTONIO_SPLIT_ENABLED' => false,
        'MONTONIO_SPLIT_CREATE_ORDER' => 'before',
        'MONTONIO_SPLIT_SHOW_LOGO' => false,
        'MONTONIO_SPLIT_MIN_ORDER_AMOUNT' => self::MIN_ORDER_TOTAL
    );

    public static function isEnabled()
    {
        return Configuration::get('MONTONIO_SPLIT_ENABLED') === '1';
    }

    public static function getCheckoutPaymentOption($module, $splitMethod)
    {
        /**
         * Check if option is enabled and supports selected currency
         */
        if (
            !self::isEnabled() ||
            !$module->isCurrencySupported(self::METHOD_CODE)
        ) {
            return null;
        }

        $context = Context::getContext();
        $cart = $context->cart;

        /**
         * Check if cart total is at least the configured minimum amount
         */
        $minCartTotal = Configuration::get('MONTONIO_SPLIT_MIN_ORDER_AMOUNT');
        $minCartTotal = is_numeric($minCartTotal) ? (float)$minCartTotal : (float)self::MIN_ORDER_TOTAL;

        if ($cart->getOrderTotal() < $minCartTotal) {
            return null;
        }

        $args = array('method' => self::METHOD_CODE);

        switch ($splitMethod) {
            case self::SPLIT_METHOD_COMBINED:
            default:
                $title               = $module->montonioSplitTitle;
                $description         = $module->montonioSplitDesc;
                $args['loan_period'] = null;
                break;
        }

        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user presses Place Order
         */
        $formAction = $context->link->getModuleLink(
            $module->name,
            'validation',
            $args,
            true
        );

        /**
         * Assign the url form action to the template var $action
         */
        $context->smarty->assign(array('action' => $formAction));

        /**
         *  Load form template to be displayed in the checkout step
         */
        $paymentForm = $module->fetch('module:montonio/views/templates/hook/payment_options.tpl');

        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($module->displayName)
            ->setCallToActionText($title)
            ->setAdditionalInformation($description)
            ->setAction($formAction)
            ->setForm($paymentForm);

        /**
         * Get Logo for Payment method and
         * assign the description to the template var $logo
         * */
        $showLogo = Configuration::get('MONTONIO_SPLIT_SHOW_LOGO');
        if ($showLogo == '1') {
            $newOption->setLogo(self::getLogoUrl());
        }

        return $newOption;
    }

    /**
     * Prepare the data to be submitted to Montonio Split
     *
     * @param object $context - The execution context
     * @param object $module - Montonio class instance
     */
    public static function preparePaymentData($context, $module)
    {
        $cart     = $context->cart;
        $customer = $context->customer;
        $address  = new Address($cart->id_address_invoice);

        // Get merchant name for multistore
        $merchantName = Configuration::get('MONTONIO_MERCHANT_NAME');
        $storePrefix  = Configuration::get('MONTONIO_ORDER_PREFIX');

        // add Prefix to Cart ID if needed
        $cartId = (string) $cart->id;
        $cartId = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);

        $notificationUrlArgs = array(
            'method'          => self::METHOD_CODE,
            'id_cart'         => $cartId,
            'id_module'       => $module->id,
            'key'             => $customer->secure_key,
            'is_notification' => 'yes',
        );

        $returnUrlArgs = array(
            'method'    => self::METHOD_CODE,
            'id_cart'   => $cartId,
            'id_module' => $module->id,
            'key'       => $customer->secure_key,
        );

        // Shipping modules' Integration ********************************************************
        $carrier     = new Carrier($cart->id_carrier);
        $carrierInfo = null;
        $carrierName = $carrier->external_module_name;
        if ($carrierName) {
            $key = $carrierName . '_' . $cart->id_address_delivery;
            if (isset($context->cookie->$key)) {
                $carrierInfo = $context->cookie->$key;
            }
        }

        if ($carrierInfo) {
            $notificationUrlArgs['extra_data'] = array($key => $carrierInfo);
            $returnUrlArgs['extra_data']       = array($key => $carrierInfo);
        }
        // /Shipping modules' Integration *******************************************************

        /**
         * Prepare Payment Data for Montonio Split
         */
        $moduleName  = $module->name;

        /**
         * Generate order before redirect
         */
        $createOrderBefore = true;
        if (Configuration::get('MONTONIO_SPLIT_CREATE_ORDER') === 'before') {
            $createOrderBefore = true;
        };
        if($createOrderBefore) {
            $module->validateOrder(
                (int)$cart->id,
                Configuration::get('PS_OS_MONTONIO'),
                $cart->getOrderTotal(),
                $module->displayName . ' (' .self::METHOD_CODE . ', Cart ID: ' . (int)$cart->id . ')',
                'Order Created with ' . self::METHOD_CODE,
                NULL,
                (int)$context->currency->id,
                false,
                $context->customer->secure_key
            );
        }
        if (isset($module->currentOrder)) {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $module->currentOrder);
            $order = new Order((int)$module->currentOrder);
        } else {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);
            $order = $context->cart;
        }
        $merchantRef = $orderRef. ((Configuration::get('MONTONIO_ENVIRONMENT') === 'sandbox') ? '_' . rand() : '');
        $orderTotal = (isset($order->total_paid)) ? $order->total_paid : $order->getOrderTotal();

        //can change on sandbox
        $returnUrlArgs['id_cart'] = $merchantRef;
        $notificationUrlArgs['id_cart'] = $merchantRef;

        $paymentData = array(
            'amount'                    => (float) $orderTotal,
            'currency'                  => 'EUR',
            'merchant_reference'        => $merchantRef,
            'merchant_name'             => $merchantName,
            'checkout_email'            => $customer->email,
            'checkout_first_name'       => $customer->firstname,
            'checkout_last_name'        => $customer->lastname,
            'checkout_phone_number'     => $address->phone,
            'checkout_city'             => $address->city,
            'checkout_address'          => $address->address1,
            'checkout_postal_code'      => $address->postcode,
            'merchant_notification_url' => $context->link->getModuleLink($moduleName, 'callback', $notificationUrlArgs),
            'merchant_return_url'       => $context->link->getModuleLink($moduleName, 'callback', $returnUrlArgs),
            'preselected_locale'        => Language::getIsoById((int) Context::getContext()->cookie->id_lang),
            'checkout_products' => array()
        );

        $paymentData['preselected_loan_period'] = Tools::getValue('loan_period', null);

        /**
         * Get products from cart and add them to data
         */
        $products = $cart->getProducts(true);

        foreach ($products as $product) {
            $paymentData['checkout_products'][] = array(
                'quantity'      => (int) $product['quantity'],
                'product_name'  => Product::getProductName($product['id_product']),
                'product_price' => 0,
            );
        }

        /**
         * Add shipping price if needed
         * @param 5 - Only Shipping
         */
        if(isset($module->currentOrder)) {
            $shippingTotal = $order->total_shipping;
        } else {
            $shippingTotal = $order->getOrderTotal(true, 5);
        }

        if ($shippingTotal > 0) {
            $paymentData['checkout_products'][] = array(
                'quantity'      => 1,
                'product_name'  => 'SHIPPING',
                'product_price' => 0,
            );
        }

        /**
         * Add Grand total to paymentData
         */
        $paymentData['checkout_products'][] = array(
            'quantity'      => 1,
            'product_name'  => (string) 'Cart/Order no: ' . $merchantRef,
            'product_price' => (float) $orderTotal,
        );

        return $paymentData;
    }

    /**
     * Get the URL where to redirect the customer to pay for the order
     *
     * @param array $paymentData The Payment Data from self::preparePaymentData
     *
     * @return string            The URL where to redirect the customer
     */
    public static function getRedirectUrl($paymentData)
    {
        $sdk = new MontonioSplitSDK(
            Configuration::get('MONTONIO_ACCESS_KEY'),
            Configuration::get('MONTONIO_SECRET_KEY'),
            Configuration::get('MONTONIO_ENVIRONMENT')
        );

        $sdk->setPaymentData($paymentData);
        return $sdk->getPaymentUrl();
    }

    // =========================================================================
    // Callback
    // =========================================================================
    public static function didUserCancel()
    {
        return !Tools::getValue('payment_token');
    }

    public static function getCallbackResponse()
    {
        $token = Tools::getValue('payment_token');

        try {
            $response = MontonioSplitSDK::decodePaymentToken($token, Configuration::get('MONTONIO_SECRET_KEY'));
            $result = array(
                'amount'             => $response->amount,
                'is_paid'            => $response->status === 'finalized',
                'access_key'         => $response->access_key,
                'merchant_reference' => $response->merchant_reference,
                'currency'           => 'EUR',
            );

            if ($response->payment_method_name) {
                $result['payment_method_name'] = $response->payment_method_name;
            }

            return $result;
        } catch (Exception $e) {
            return null;
        }
    }

    /**
     * Get smarty data for legacy checkout option (Prestashop 1.6)
     *
     * @param array$params The params from hookPayment hook
     * @return array       The smarty params
     */
    public static function getLegacyPaymentOption($module, $splitMethod, $params)
    {

        /**
         * Check if cart total is between the configured min and max
         */
        $min   = Configuration::get('MONTONIO_SPLIT_MIN_ORDER_AMOUNT');
        $total = $params['cart']->getOrderTotal();

        $min   = is_numeric($min) ? (float) $min : (float)self::MIN_ORDER_TOTAL;
        $total = is_numeric($total) ? (float) $total : null;

        $limitCheck = true;
        if (is_float($total)) {
            $limitCheck = $min <= $total;
        }

        $enabled = self::isEnabled() && $module->isCurrencySupported(self::METHOD_CODE) && $limitCheck;

        $montonioSplitData = array(
            'enabled'            => $enabled,
            'title'              => $module->montonioSplitTitle,
            'description'        => $module->montonioSplitDesc,
            'loan_period'        => null,
            'logo_url'           => self::getLogoUrl(),
            'show_logo'          => Configuration::get('MONTONIO_SPLIT_SHOW_LOGO') == '1'
        );

        return $montonioSplitData;
    }

    /**
     *
     * @return string
     */
    public static function getLogoUrl()
    {
        $logos = array(
            'et' => 'https://public.montonio.com/logo/financing/ee-split-3.png',
            'lv' => 'https://public.montonio.com/logo/financing/lt-split-3.png',
            'lt' => 'https://public.montonio.com/logo/financing/lv-split-3.png',
            'en' => 'https://public.montonio.com/logo/financing/en-split-3.png'
        );
        $lang = Context::getContext()->language->iso_code;
        return isset($logos[$lang]) ? $logos[$lang] : $logos['en'];
    }
}
