<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioFinancing.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioBlikPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioCardPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioSplit.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/PaymentLock.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/PaymentStatus.php';


class Montonio extends PaymentModule
{
    public $montonioPaymentsTitle;
    public $montonioFinancingTitle;
    public $montonioFinancingDesc;
    public $montonioSplitTitle;
    public $montonioSplitDesc;
    public $montonioCardPaymentsTitle;
    public $montonioCardPaymentsDesc;
    const MONTONIO_PAYMENTS_SUPPORTED_CURRENCIES  = array('EUR', 'PLN');
    const MONTONIO_BLIK_SUPPORTED_CURRENCIES  = array('PLN');
    const MONTONIO_FINANCING_SUPPORTED_CURRENCIES = array('EUR');

    const NUM_MONTONIO_METHODS = 5;
    const MONTONIO_PAYMENT_METHODS = array(
        'MontonioPayments',
        'MontonioFinancing',
        'MontonioSplit',
        'MontonioBlikPayments',
        'MontonioCardPayments'
    );

    public $paymentTranslations;

    public $textRegionEE;
    public $textRegionLV;
    public $textRegionLT;
    public $textRegionFI;
    public $textRegionPL;
    public $textRegionDE;

    public $textCardPaymentsTitle;
    public $textCardPaymentsDescription;

    public function __construct()
    {
        $this->name             = 'montonio';
        $this->tab              = 'payments_gateways';
        $this->version          = '1.4.7';
        $this->author           = 'Montonio Finance OÜ';
        $this->is_eu_compatible = 1;
        $this->need_instance    = 0;
        $this->module_key       = '0920be4967c55d1907951c05fbbdacfc';
        $this->bootstrap        = true;
        $this->displayName            = 'Montonio';
        $this->description            = $this->l('Montonio services for Prestashop');
        $this->confirmUninstall       = $this->l('Are you sure you want to uninstall Montonio?');
        $this->limited_currencies     = array('EUR', 'PLN');
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        parent::__construct();

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->moduleConfig           = array();

        $this->montonioPaymentsTitle = $this->l('Pay with your bank', 'montonio');
        $this->montonioPaymentsDesc  = $this
            ->l('Complete the purchase with a direct payment from your bank account.', 'montonio');

        $this->montonioBlikPaymentsTitle = $this->l('Pay with BLIK', 'montonio');
        $this->montonioBlikPaymentsDesc  = $this
            ->l('Complete the purchase with BLIK.', 'montonio');

        $this->montonioCardPaymentsTitle = $this->l('Card Payment', 'montonio');
        $this->montonioCardPaymentsDesc  = $this
            ->l('Pay with your credit or debit card via Montonio.', 'montonio');

        $this->montonioFinancingTitle = $this->l('Compare financing offers', 'montonio');
        $this->montonioFinancingDesc  = $this
        ->l('Select the financing offer you like best! Receive multiple financing offers with one application.', 'montonio');

        $this->montonioSplitTitle = $this->l('Pay later, no fees', 'montonio');
        $this->montonioSplitDesc  = $this
        ->l('Pay later in up to three payments without any additional fees. 0% interest, 0€ contract fee.', 'montonio');

        // Translatable strings
        $this->textRegionEE                = $this->l('Estonia', 'montonio');
        $this->textRegionLT                = $this->l('Lithuania', 'montonio');
        $this->textRegionLV                = $this->l('Latvia', 'montonio');
        $this->textRegionFI                = $this->l('Finland', 'montonio');
        $this->textRegionPL                = $this->l('Poland', 'montonio');
        $this->textRegionDE                = $this->l('Germany', 'montonio');
        $this->textCardPaymentsTitle       = $this->l('Card Payment', 'montonio');
        $this->textBlikPaymentsTitle       = $this->l('BLIK Payment', 'montonio');
        $this->textCardPaymentsDescription = $this->l('Pay with your credit or debit card via Montonio', 'montonio');

    }

    /**
     * Call user_func_array on supported payment method
     *
     * @param string $paymentMethod The payment method name in self::MONTONIO_PAYMENT_METHODS
     * @param string $functionName The static method to be called from the $paymentMethod
     * @param mixed ...$args The arguments passed to the function
     *
     * @return mixed                The response from the Montonio Payment Method function
     * @throws Exception
     */
    public function callPaymentFunc($paymentMethod, $functionName, ...$args)
    {
        if (!in_array($paymentMethod, self::MONTONIO_PAYMENT_METHODS)) {
            throw new \Exception('Unsupported payment method.');
        }

        return call_user_func($paymentMethod . '::' . $functionName, ...$args);
    }

    /**
     * Install this module and register the following Hooks:
     * [payment, displayProductPriceBlock]
     *
     * @return bool
     */
    public function install()
    {
        if (!parent::install()) {
            $this->_errors[] = $this->l('Parent install failed');
            return false;
        }

        /**
         * Register Prestashop hooks
         */
        if (!$this->registerHook('payment')
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayProductPriceBlock')
        ) {
            return false;
        }

        /**
         * Update shop configuration
         */
        $shops = Shop::getShops();

        foreach (self::getDefaultConfig() as $key => $value) {
            if (Shop::isFeatureActive()) {
                foreach ($shops as $shop) {
                    if (!Configuration::updateValue($key, $value, false, null, (int) $shop['id_shop'])) {
                        return false;
                    }
                }
            } else {
                if (!Configuration::updateValue($key, $value)) {
                    return false;
                }
            }
        }

        /**
         * add order status
         */

        $status = new PaymentStatus();
        if(is_object($status)) {
            $status->addStatus();
        }

        /**
         * Default order placement mode
         */

        Configuration::updateValue('MONTONIO_PAYMENTS_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_FINANCING_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_SPLIT_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_BLIK_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_CARD_CREATE_ORDER', 'before');

        return $this->installPaymentLockTable();
    }

    /**
     * Method to uninstall Montonio
     *
     * @return boolean
     */
    public function uninstall()
    {
        /**
         * Delete config values on uninstall (except for order state)
         */
        foreach (array_keys(self::getDefaultConfig()) as $key) {
            if (!Configuration::deleteByName($key)) {
                return false;
            }
        }

        return parent::uninstall();
    }

    public function installPaymentLockTable()
    {
        try {
            $paymentLock = new PaymentLock();
            $paymentLock->installResponseLockTable();
            return true;
        } catch (Exception $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }
    }

    // =========================================================================
    // Prestashop Hooks
    // =========================================================================

    public function hookDisplayHeader()
    {
        /*
         * Verify this module is active
         */
        if (!$this->active) {
            return;
        }

        $paymentHandleStylesheet = Configuration::get('MONTONIO_PAYMENTS_STYLE');
        if (!empty($paymentHandleStylesheet)) {
            $this->context->controller->addCSS(
                _PS_MODULE_DIR_ . 'montonio/views/css/payment-handle/' . $paymentHandleStylesheet . '.css',
                'all'
            );
        }
        $this->context->controller->addJS(_PS_MODULE_DIR_ . 'montonio/views/js/montonio-payment-handle.js');
    }

    /**
     * Display loan calculator at product page
     *
     * @return void
     */
    public function hookDisplayProductPriceBlock($params)
    {

        if ($params['type'] !== 'price') {
            return;
        }
        if (Configuration::get('MONTONIO_FINANCING_CALC_ENABLED') !== 'yes' ||
            Configuration::get('MONTONIO_FINANCING_ENABLED') !== '1'
        ) {
            return;
        }
        if (!isset($params['product']->id)) {
            return;
        }

        $productPrice = Product::getPriceStatic($params['product']->id);

        /**
         * Get options for loan calculator
         */
        $calcData = array(
            'data-product-price' => (string) $productPrice,
            'data-access-key'    => Configuration::get('MONTONIO_ACCESS_KEY'),
            'data-environment'   => Configuration::get('MONTONIO_ENVIRONMENT'),
            'data-name'          => $this->montonioFinancingTitle,
            'data-css'           => Configuration::get('MONTONIO_FINANCING_CALC_CSS'),
            'data-locale'        => $this->getMontonioLocaleCode(),
            'data-logo-url'      => Configuration::get('MONTONIO_FINANCING_LOGO_URL'),
            'data-show-modal'    => Configuration::get('MONTONIO_FINANCING_CALC_SHOW_MODAL') == 'yes'
            ? 'true'
            : 'false',
            'data-info-page-url' => Configuration::get('MONTONIO_FINANCING_CALC_INFO_PAGE_URL'),
        );

        /**
         * Add data to product page and show calculator block
         */
        $this->smarty->assign(['montonioCalculatorData' => $calcData]);
        return $this->display(__FILE__, 'loan_calculator.tpl');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {

        /*
         * Verify if this module is active
         */
        if (!$this->active) {
            return;
        }

        $currency_id = $params['cart']->id_currency;
        $currency    = new Currency((int) $currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }

        $logo = 'https://public.montonio.com/logo/montonio_dark_icononly.png';
        $this->smarty->assign('module_dir', $this->_path);
        $this->smarty->assign('montonio_checkout_logo', $logo);

        $montonioFinancingData = $this->callPaymentFunc('MontonioFinancing', 'getLegacyPaymentOption', $this, $params);
        $montonioPaymentsData = $this->callPaymentFunc('MontonioPayments', 'getLegacyPaymentOption', $this);
        $montonioCardPaymentsData = $this->callPaymentFunc('MontonioCardPayments', 'getLegacyPaymentOption', $this);
        $montonioBlikPaymentsData = $this->callPaymentFunc('MontonioBlikPayments', 'getLegacyPaymentOption', $this);

        $montonioSplitData = $this->callPaymentFunc(
            'MontonioSplit',
            'getLegacyPaymentOption',
            $this,
            MontonioSplit::SPLIT_METHOD_COMBINED,
            $params
        );

        // $montonioPaymentsData = array(
        //     'enabled' => Configuration::get('MONTONIO_PAYMENTS_ENABLED') == '1',
        //     'style' => Configuration::get('MONTONIO_PAYMENTS_STYLE'),
        //     'banks' => json_decode(Configuration::get('MONTONIO_PAYMENTS_BANK_LIST'))
        // );
        $this->smarty->assign(array('montonioFinancingData' => $montonioFinancingData));
        $this->smarty->assign(array('montonioPaymentsData' => $montonioPaymentsData));
        $this->smarty->assign(array('montonioBlikPaymentsData' => $montonioBlikPaymentsData));
        $this->smarty->assign(array('montonioCardPaymentsData' => $montonioCardPaymentsData));
        $this->smarty->assign(array('montonioSplitData' => $montonioSplitData));

        $paymentMethodsOrder = array();
        for ($i = 1; $i <= self::NUM_MONTONIO_METHODS; $i++) {
            $paymentMethodsOrder[] = Configuration::get('MONTONIO_METHOD_' . $i . '_TYPE');
        }

        $this->smarty->assign(array('paymentMethodsOrder' => $paymentMethodsOrder));

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * Get default configuration
     *
     * @return array
     */
    protected static function getDefaultConfig()
    {
        $config = array(
            'MONTONIO_ACCESS_KEY'                       => '',
            'MONTONIO_SECRET_KEY'                       => '',
            'MONTONIO_ENVIRONMENT'                      => '',
            'MONTONIO_ORDER_PREFIX'                     => '',
            'MONTONIO_MERCHANT_NAME'                    => '',
            'MONTONIO_ADVANCED_ORDER_CONFIRMATION_PAGE' => 'order-detail',
        );

        foreach (self::MONTONIO_PAYMENT_METHODS as $method) {
            $config = array_merge($config, constant($method . '::CONFIGDEFAULTS'));
        }

        for ($i = 1; $i <= self::NUM_MONTONIO_METHODS; $i++) {
            $config = array_merge($config, array(
                'MONTONIO_METHOD_' . $i . '_TYPE' => 'none'
            ));
        }

        $config['MONTONIO_METHOD_1_TYPE'] = 'MontonioPayments';
        $config['MONTONIO_METHOD_2_TYPE'] = 'MontonioFinancing';
        $config['MONTONIO_METHOD_3_TYPE'] = 'MontonioSplit1';
        $config['MONTONIO_METHOD_4_TYPE'] = 'MontonioCardPayments';
        $config['MONTONIO_METHOD_5_TYPE'] = 'MontonioBlikPayments';

        return $config;
    }

    /**
     * Use Tools::getValue on config keys for each Montonio Payment Method
     *
     * @return array
     */
    protected static function getSubmittedConfig()
    {
        $config = self::getDefaultConfig();
        array_walk($config, function (&$a, $b) {
            $a = Tools::getValue($b);
        });
        return $config;
    }

    /**
     * Use Configuration::get on config keys for each Montonio Payment Method
     *
     * @return array
     */
    protected static function getCurrentConfig()
    {
        $config = self::getDefaultConfig();
        array_walk($config, function (&$a, $b) {
            $a = Configuration::get($b);
        });
        return $config;
    }

    /**
     * Undocumented function
     *
     * @param array $settings - New Admin Settings data
     * @return string - the JSON-encoded banklist array
     */
    protected function syncBanks($settings)
    {
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

        $token = MontonioPaymentsSDK::getBearerToken(
            $settings['MONTONIO_ACCESS_KEY'],
            $settings['MONTONIO_SECRET_KEY']
        );

        $env    = $settings['MONTONIO_ENVIRONMENT'];
        $apiUrl = $env === 'sandbox'
        ? 'https://api.sandbox-payments.montonio.com/pis/merchants/aspsps'
        : 'https://api.payments.montonio.com/pis/merchants/aspsps';

        $options = array(
            'http' => array(
                'header' => "Content-Type: application/json\r\n" .
                "Authorization: Bearer " . $token . "\r\n",
                'method' => 'GET',
            ),
        );

        $context = stream_context_create($options);
        $result  = Tools::file_get_contents($apiUrl, false, $context);

        // error handling
        if ($result === false) {
            return false;
        } else {
            return $result;
        }
    }

    // =========================================================================
    // Admin options and Configuration
    // =========================================================================

    /**
     * Returns a string containing the HTML necessary to
     * generate a configuration screen on the admin
     *
     * @return string
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $newConfig = self::getSubmittedConfig();

            if (!$newConfig['MONTONIO_ACCESS_KEY'] || !$newConfig['MONTONIO_SECRET_KEY']) {
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            } else {
                if (Tools::getValue('MONTONIO_PAYMENTS_ENABLED') == '1') {
                    if (Tools::getValue('MONTONIO_ENVIRONMENT') == 'sandbox') {
                        $newConfig['MONTONIO_PAYMENTS_CURRENT_BANKS_SANDBOX'] =
                            json_encode($this->fetchBanklist($newConfig));
                    }

                    if (Tools::getValue('MONTONIO_ENVIRONMENT') == 'production') {
                        $newConfig['MONTONIO_PAYMENTS_CURRENT_BANKS'] = json_encode($this->fetchBanklist($newConfig));
                    }
                }

                foreach ($newConfig as $key => $value) {
                    Configuration::updateValue($key, $value);
                }

                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        return $output . $this->displayForm();
    }

    /**
     * The Admin Configuration for Montonio
     *
     * @return void
     */
    public function displayForm()
    {

        /**
         * Enqueue the admin stylesheet
         */
        $this->context->controller->addCSS(
            _PS_MODULE_DIR_ . 'montonio/views/css/admin/configuration.css',
            'all'
        );

        /**
         * Get default language
         */
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        /**
         * Init form fields for admin
         */
        $fields_form            = array();
        $fields_form[0]['form'] = array(
            'tinymce' => true,
            'legend'  => array(
                'title' => $this->l('General Settings'),
                'icon'  => 'icon-cogs',
            ),
            'input'   => array(
                array(
                    'type'     => 'select',
                    'label'    => $this->l('Environment'),
                    'hint'     => $this->l('Use sandbox mode for development only'),
                    'name'     => 'MONTONIO_ENVIRONMENT',
                    'required' => 'true',
                    'options'  => array(
                        'query' => array(
                            array(
                                'id'   => 'production',
                                'name' => $this->l('Production'),
                            ),
                            array(
                                'id'   => 'sandbox',
                                'name' => $this->l('Sandbox'),
                            ),
                        ),
                        'id'    => 'id',
                        'name'  => 'name',
                    ),
                ),
                array(
                    'type'     => 'text',
                    'label'    => $this->l('Montonio Access Key'),
                    'name'     => 'MONTONIO_ACCESS_KEY',
                    'required' => true,
                ),
                array(
                    'type'     => 'text',
                    'label'    => $this->l('Secret key'),
                    'desc'     => 'Montonio Public and Secret keys can be obtained from the integrations tab in '
                    . '<a href="https://partner.montonio.com" target="_blank">Montonio Partner System</a>'
                    . ' and Sandbox keys from <a href="https://sandbox-partner.montonio.com" target="_blank">'
                    . 'Montonio Sandbox Partner System</a>',
                    'name'     => 'MONTONIO_SECRET_KEY',
                    'required' => true,
                ),
                array(
                    'type'     => 'text',
                    'label'    => $this->l('Order Prefix'),
                    'desc'     => '<strong>[MULTISTORE]</strong>If you are using Montonio with only one pair of API '
                    . 'keys, you can set the Order Prefix to distinguish between orders in the Montonio Partner System',
                    'name'     => 'MONTONIO_ORDER_PREFIX',
                    'required' => false,
                ),
                array(
                    'type'     => 'text',
                    'label'    => $this->l('Merchant name'),
                    'desc'     => '<strong>[MULTISTORE]</strong>The name to be shown to customers in Montonio '
                    . 'applications. By default this is '
                    . 'your company\'s Display Name (set in the Partner System), but you can also override it here',
                    'name'     => 'MONTONIO_MERCHANT_NAME',
                    'required' => false,
                ),
                array(
                    'type'     => 'select',
                    'label'    => $this->l('Order Confirmation page'),
                    'hint'     => $this->l('Allows to control the page you are redirected after order placement'),
                    'name'     => 'MONTONIO_ADVANCED_ORDER_CONFIRMATION_PAGE',
                    'required' => 'true',
                    'options'  => array(
                        'query' => array(
                            array(
                                'id'   => 'order-confirmation',
                                'name' => $this->l('Order Confirmation page'),
                            ),
                            array(
                                'id'   => 'order-detail',
                                'name' => $this->l('Order details page'),
                            ),
                        ),
                        'id'    => 'id',
                        'name'  => 'name',
                    ),
                ),
            ),
            'submit'  => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ),
        );

        foreach (self::MONTONIO_PAYMENT_METHODS as $key => $method) {
            $fields_form[]['form'] = array(
                'tinymce' => true,
                'legend'  => array(
                    'title' => $this->l($method),
                    'icon'  => 'icon-cogs',
                ),
                'input'   => $this->callPaymentFunc($method, 'getConfig'),
                'submit'  => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ),
            );
        }

        $paymentMethodsInput = array();
        for ($i = 1; $i < self::NUM_MONTONIO_METHODS + 1; $i++) {
            array_push($paymentMethodsInput, array(
                'type'     => 'select',
                'label'    => 'Payment method ' . $i,
                'name'     => 'MONTONIO_METHOD_' . $i . '_TYPE',
                'required' => 'true',
                'options'  => array(
                    'query' => array(
                        array(
                            'id'   => 'none',
                            'name' => 'Not Used',
                        ),
                        array(
                            'id'   => 'MontonioPayments',
                            'name' => 'Montonio Payments',
                        ),
                        array(
                            'id'   => 'MontonioCardPayments',
                            'name' => 'Montonio Card Payments',
                        ),
                        array(
                            'id'   => 'MontonioFinancing',
                            'name' => 'Montonio Financing',
                        ),
                        array(
                            'id'   => 'MontonioSplit',
                            'name' => 'Montonio Split Combined',
                        ),
                        array(
                            'id' => 'MontonioBlikPayments',
                            'name' => 'Montonio BLIK Payments',
                        )
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                )
            ));
        }

        $fields_form[]['form'] = array(
            'tinymce' => true,
            'legend'  => array(
                'title' => 'Payment method order',
                'icon'  => 'icon-cogs',
            ),
            'input'   => $paymentMethodsInput,
            'submit'  => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ),
        );

        /**
         * Setup advanced fields
         */
        $fields_form[]['form'] = array(
            'tinymce' => true,
            'legend'  => array(
                'title' => $this->l('Advanced Settings', 'montonio'),
                'icon'  => 'icon-cogs',
            ),
            'input'   => array(
                array(
                    'type'     => 'select',
                    'label'    => 'Order Confirmation page',
                    'desc'     => '<strong>[ADVANCED]</strong> This controls which page to redirect to
                    after a successful payment.<br/>
                    <i>NB! In some older versions of PrestaShop, the Order Confirmation page might not exist!</i>',
                    'name'     => 'MONTONIO_ADVANCED_ORDER_CONFIRMATION_PAGE',
                    'required' => 'true',
                    'options'  => array(
                        'query' => array(
                            array(
                                'id'   => 'order-detail',
                                'name' => 'Order Detail',
                            ),
                            array(
                                'id'   => 'order-confirmation',
                                'name' => 'Order Confirmation',
                            ),
                        ),
                        'id'    => 'id',
                        'name'  => 'name',
                    ),
                ),
            ),
            'submit'  => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ),
        );

        /**
         * Setup admin options
         */
        $helper                           = new HelperForm();
        $helper->module                   = $this;
        $helper->name_controller          = $this->name;
        $helper->token                    = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex             = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language    = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->title                    = $this->displayName;
        $helper->show_toolbar             = true;
        $helper->toolbar_scroll           = true;
        $helper->submit_action            = 'submit' . $this->name;
        $helper->toolbar_btn              = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ),
        );

        /**
         * Fill the inputs with current data
         */
        $config = self::getCurrentConfig();
        foreach ($config as $key => $value) {
            $helper->fields_value[$key] = $value;
        }

        return $helper->generateForm($fields_form);
    }

    // =========================================================================
    // Helpers
    // =========================================================================

    /**
     * Validate currency
     *
     * @return boolean True if supported, false otherwise
     */
    public function isCurrencySupported($method)
    {
        try {
            $idCurrency = $this->context->cart->id_currency;

            $paymentCurrency = new Currency($idCurrency); // should contain iso_code
            switch ($method) {
                case 'MontonioFinancing':
                case 'MontonioSplit':
                    $supportedCurrencies = self::MONTONIO_FINANCING_SUPPORTED_CURRENCIES;
                    break;
                case 'MontonioBlikPayments':
                    $supportedCurrencies = self::MONTONIO_BLIK_SUPPORTED_CURRENCIES;
                    break;
                case 'MontonioPayments':
                case 'MontonioCardPayments':
                    $supportedCurrencies = self::MONTONIO_PAYMENTS_SUPPORTED_CURRENCIES;
                    break;
                default:
                    $supportedCurrencies = array();
            }
            return (isset($paymentCurrency->iso_code)
                && in_array($paymentCurrency->iso_code, $supportedCurrencies));
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Get Montonio Locale
     *
     * @return string
     */
    protected function getMontonioLocaleCode()
    {
        $iso   = $this->context->language->iso_code;
        $langs = array(
            'lt' => 'lt',
            'et' => 'et',
            'ru' => 'ru',
            'en' => 'en',
            'pl' => 'pl',
            'de' => 'de'
        );

        return isset($langs[$iso]) ? $langs[$iso] : 'en';
    }

    private function fetchBanklist($config)
    {
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

        $result = array();
        $sdk = new MontonioPaymentsSDK(
            $config['MONTONIO_ACCESS_KEY'],
            $config['MONTONIO_SECRET_KEY'],
            $config['MONTONIO_ENVIRONMENT']
        );

        $response = $sdk->fetch_payment_methods();

        if ($response['status'] == 'SUCCESS') {
            $result = $response['data'];
        }

        return $result;
    }
}
